////////////////////////////////////////////////////////////////////////////////
//       Runge-Kutt Method of solving of dif. equations and their systems     //
//                        Runge-Kutt Method v.1.00 Eng                        //
//                                                                            //
//                        Author - Andrey G. Sadovoy                          //
//                                                                            //
//                               dec. 2005                                    //
//                                                                            //
// (c) All Rights Reserved 2005                                               //
////////////////////////////////////////////////////////////////////////////////

unit RK_Method;

interface

type
        TVarsArray = array of Extended; // array of vars with undepended var
        TInitArray = array of Extended; // array of initial conditions
        TFunArray = array of function(VarsArray: TVarsArray ):Extended;
        // array of functions
        TResArray = array of array of Extended; // matrix of results
        TCoefsArray = array of Extended; // array of method coef.

function Runge_Kutt(  // Runge-Kutt method
        FunArray: TFunArray; // array of functions
        First: Extended; // begining point of undepended axis
        Last: Extended; // end point of undepended axis
        Steps: Integer; // number of steps
        InitArray: TInitArray; // initial conditions array
        var Res: TResArray // matrix of results including undepended var
         ):Word; // output - error code

implementation

Function Runge_Kutt(FunArray: TFunArray; First: Extended; Last: Extended;
Steps: Integer; InitArray: TInitArray; var Res: TResArray):Word;
var
        Num: Word; // number of equations
        NumInit: Word; // number of initial conditions
        Delt: Extended; // length of one step
        Vars: TVarsArray; // array of vars including undepended var
        Vars2,Vars3,Vars4: TVarsArray; // vars for 2-4 th coefs.
        Coefs1: TCoefsArray; // array of 1 method coefs
        Coefs2: TCoefsArray; // array of 2 method coefs
        Coefs3: TCoefsArray; // array of 3 method coefs
        Coefs4: TCoefsArray; // array of 4 method coefs
        I: Integer; // itterations
        J: Word; // method coefs index
        K: Integer; // other
begin
 Num:=Length(FunArray); // Get amount of equations
 NumInit:=Length(InitArray); // Get amount of initial conditions
 If NumInit<>Num then
   begin
     Result:=100; // Error Code 100: amount of equations are not equal to
                  // amount of initial conditions
     Exit;
   end;
 Delt:=(Last-First)/Steps; // Get length of step
 SetLength(Res,Num+1,Steps+1);//Set size for result matrix (incl. undep. var)
 SetLength(Vars,Num+1); // Set size for vars array including undep. var
 SetLength(Vars2,Num+1); // amount of vars for 2-nd coef. (incl. undep. var)
 SetLength(Vars3,Num+1); // amount of vars for 3 coef. (incl. undep. var)
 SetLength(Vars4,Num+1); // amount of vars for 4 coef. (incl. undep. var)
 SetLength(Coefs1,Num); // amount 1 coefs. = amount of equations
 SetLength(Coefs2,Num); // amount 2 coefs. = amount of equations
 SetLength(Coefs3,Num); // amount 3 coefs. = amount of equations
 SetLength(Coefs4,Num); // amount 4 coefs. = amount of equations
 // Initial values of vars:
 Vars[0]:=First;
 For K:=0 to NumInit-1 do Vars[K+1]:=InitArray[K];
 For J:=0 to Num do Res[J,0]:=Vars[J]; // first point of result
 For I:=0 to Steps-1 do // itterations begin
        begin
           For J:=0 to Num-1 do Coefs1[J]:=FunArray[J](Vars)*delt; // Coef. #1
           // Get values of vars for second Coef. :
           Vars2[0]:=Vars[0]+delt/2;
           For K:=1 to Num do Vars2[K]:=Vars[K]+Coefs1[K-1]/2;
           For J:=0 to Num-1 do Coefs2[J]:=FunArray[J](Vars2)*delt; // Coef. #2
           // Get values of vars for Coef. #3  :
           Vars3[0]:=Vars[0]+delt/2;
           For K:=1 to Num do Vars3[K]:=Vars[K]+Coefs2[K-1]/2;
           For J:=0 to Num-1 do Coefs3[J]:=FunArray[J](Vars3)*delt; // Coef. #3
           // Get values of vars for Coef. #4  :
           Vars4[0]:=Vars[0]+delt;
           For K:=1 to Num do Vars4[K]:=Vars[K]+Coefs3[K-1];
           For J:=0 to Num-1 do Coefs4[J]:=FunArray[J](Vars4)*delt; // Coef. #4
           // Get new values of vars including undepended var :
           Vars[0]:=Vars[0]+delt;
           For K:=1 to Num do
           Vars[K]:=Vars[K]+
           (1/6)*(Coefs1[K-1]+2*(Coefs2[K-1]+Coefs3[K-1])+Coefs4[K-1]);
           // Itteration result:
           For J:=0 to Num do Res[J,I+1]:=Vars[J];
        end; // Itterations end
 Result:=0; // Error Code 0 - No Errors
end;


end.
